// $Id: $
// Include files 

// from Gaudi
#include "GaudiKernel/DeclareFactoryEntries.h" 

// local
#include "ALinkReader.h"
#include "DecodeConf.h"
#include "DecodeCore.h"

using namespace VeloFullConf;
using namespace DecodeCore;

//-----------------------------------------------------------------------------
// Implementation file for class : ALinkReader
//
// 2006-04-24 : Tomasz Szumlak
//-----------------------------------------------------------------------------

// Declaration of the Algorithm Factory
DECLARE_ALGORITHM_FACTORY( ALinkReader );

//=============================================================================
// Standard constructor, initializes variables
//=============================================================================
ALinkReader::ALinkReader( const std::string& name,
                          ISvcLocator* pSvcLocator)
  : GaudiAlgorithm ( name , pSvcLocator ),
    m_decodedADCLoc ( VeloTELL1DataLocation::ADCs ),
    m_decodedPedLoc ( VeloTELL1DataLocation::Pedestals ),
    m_decodedHeaderLoc ( VeloTELL1DataLocation::Headers ),
    m_evtInfoLoc ( EvtInfoLocation::Default )
  { }
//=============================================================================
// Destructor
//=============================================================================
ALinkReader::~ALinkReader() {}
//=============================================================================
// Initialization
//=============================================================================
StatusCode ALinkReader::initialize() {
  StatusCode sc = GaudiAlgorithm::initialize(); // must be executed first
  if ( sc.isFailure() ) return sc;  // error printed already by GaudiAlgorithm

  debug() << "==> Initialize" << endmsg;
  //
  return StatusCode::SUCCESS;
}

//=============================================================================
// Main execution
//=============================================================================
StatusCode ALinkReader::execute() {

  debug() << "==> Execute" << endmsg;
  //
  getData();
  getAllSensors();
  getOneSensor();
  //
  return ( StatusCode::SUCCESS );
}

//=============================================================================
//  Finalize
//=============================================================================
StatusCode ALinkReader::finalize() {

  debug() << "==> Finalize" << endmsg;

  return GaudiAlgorithm::finalize();  // must be called after all other actions
}

//=============================================================================
StatusCode ALinkReader::getData()
{
  debug()<< " ==> getData() " <<endmsg;
  //
  if(!exist<VeloTELL1Datas>(m_decodedADCLoc)){
    info()<< " ==> There is no decoded ADCs at: "
           << m_decodedADCLoc <<endmsg;
    return ( StatusCode::FAILURE );
  }else{  
    // get data banks from default TES location
    m_decodedADCs=get<VeloTELL1Datas>(m_decodedADCLoc);
    debug()<< " ==> The data banks have been read-in from location: "
           << m_decodedADCLoc
           << ", size of data container (number of read-out TELL1s): "
           << m_decodedADCs->size() <<endmsg;
  }
  //
  if(!exist<VeloTELL1Datas>(m_decodedPedLoc)){
    info()<< " ==> There is no decoded ADCs at: "
           << m_decodedPedLoc <<endmsg;
    return ( StatusCode::FAILURE );
  }else{  
    // get data banks from default TES location
    m_decodedPeds=get<VeloTELL1Datas>(m_decodedPedLoc);
    debug()<< " ==> The data banks have been read-in from location: "
           << m_decodedPedLoc
           << ", size of data container (number of read-out TELL1s): "
           << m_decodedPeds->size() <<endmsg;
  }
  //
  if(!exist<VeloTELL1Datas>(m_decodedHeaderLoc)){
    info()<< " ==> There is no decoded ADCs at: "
           << m_decodedHeaderLoc <<endmsg;
    return ( StatusCode::FAILURE );
  }else{  
    // get data banks from default TES location
    m_decodedHeaders=get<VeloTELL1Datas>(m_decodedHeaderLoc);
    debug()<< " ==> The data banks have been read-in from location: "
           << m_decodedHeaderLoc
           << ", size of data container (number of read-out TELL1s): "
           << m_decodedHeaders->size() <<endmsg;
  }
  //
  if(!exist<EvtInfos>(m_evtInfoLoc)){
    info()<< " ==> There is no event info at: "
           << m_evtInfoLoc <<endmsg;
    return ( StatusCode::FAILURE );
  }else{  
    // get data banks from default TES location
    m_evtInfos=get<EvtInfos>(m_evtInfoLoc);
    debug()<< " ==> The data banks have been read-in from location: "
           << m_evtInfoLoc
           << ", size of data container (number of read-out TELL1s): "
           << m_evtInfos->size() <<endmsg;
  }
  //
  return ( StatusCode::SUCCESS );
}
//
StatusCode ALinkReader::getAllSensors()
{
  // declare iterator for data object with ADC samples 
  // for each sensor
  VeloTELL1Datas::const_iterator sensIt;
  // vector for 32 channels 
  dataVec ALink;
  // declare iterator for navigate through 
  // vector with channels
  cdatIt aLinkIt;
  // loop over sensor data
  for(sensIt=m_decodedADCs->begin(); sensIt!=m_decodedADCs->end(); sensIt++){
    // temporary object for ADCs 
    VeloTELL1Data* adcData=(*sensIt);
    const int TELL1No=adcData->key();
    EvtInfo* anInfo=m_evtInfos->object(TELL1No);
    debug()<< "detectorID: " << anInfo->detectorID() <<endmsg;
    debug()<< "TELL1 number: " << TELL1No <<endmsg;
    // get ALinks
    for(int ALinkCounter=0; ALinkCounter<NumberOfALinks; ALinkCounter++){
      // returns vector with 32 channels
      ALink=(*adcData)[ALinkCounter];
      debug()<< " Data for ALink: ["
            << ALinkCounter << "]" <<endmsg;
      int channelCounter=0;
      // check output
      if(ALinkCounter==0){
        for(aLinkIt=ALink.begin(); aLinkIt!=ALink.end(); aLinkIt++){
          debug()<< " ADC[" << ALinkCounter << "][" 
                << channelCounter << "]" << (*aLinkIt) <<endmsg;
          channelCounter++;
        }
      }
    }
  }
  // get pedestals now
  for(sensIt=m_decodedPeds->begin(); sensIt!=m_decodedPeds->end(); sensIt++){
    VeloTELL1Data* pedData=(*sensIt);
    const int TELL1No=pedData->key();
    debug()<< " TELL1 number: " << TELL1No <<endmsg;
    for(int ALinkCounter=0; ALinkCounter<NumberOfALinks; ALinkCounter++){
      ALink=(*pedData)[1];
      int channelCounter=0;
      if(ALinkCounter==1){
        debug()<< "ALink: " << ALinkCounter <<endmsg;
        for(aLinkIt=ALink.begin(); aLinkIt!=ALink.end(); aLinkIt++){
          debug()<< " Ped[" << ALinkCounter << "][" 
                << channelCounter << "]" << (*aLinkIt) <<endmsg;
          channelCounter++;
        }
      }
    }
  }
  // check headers
  for(sensIt=m_decodedHeaders->begin();
      sensIt!=m_decodedHeaders->end(); sensIt++){
    VeloTELL1Data* headData=(*sensIt);
    const int TELL1No=headData->key();
    debug()<< " TELL1 number: " << TELL1No <<endmsg;
    for(int ALinkCounter=0; ALinkCounter<NumberOfALinks; ALinkCounter++){
      ALink=(*headData)[16];
      int channelCounter=0;
      if(ALinkCounter==16){
        debug()<< "ALink: " << ALinkCounter <<endmsg;
        for(aLinkIt=ALink.begin(); aLinkIt!=ALink.end(); aLinkIt++){
          debug()<< " Header[" << ALinkCounter << "][" 
                << channelCounter << "]" << (*aLinkIt) <<endmsg;
          channelCounter++;
        }
      }
    }
  }
  //
  return ( StatusCode::SUCCESS );
}
//
StatusCode ALinkReader::getOneSensor()
{
  // get one TELL1 data and event info section for the
  // sensor, next get ADC headers 
  const int aSensor=21;
  VeloTELL1Data* aSensorData=m_decodedADCs->object(aSensor);
  // check number of sensor/TELL1 for the given number
  info()<< " Number of the TELL1: " << aSensorData->key() <<endmsg;
  // get event info section for the sensor
  EvtInfo* aSensorInfo=m_evtInfos->object(aSensorData->key());
  // print some infos
  for(int PPFPGA=0; PPFPGA<NumberOfPPFPGA; PPFPGA++){    
    unsigned int bunchCounter=aSensorInfo->bunchCounter(PPFPGA);
    unsigned int detectorID=aSensorInfo->detectorID(PPFPGA);
    unsigned int check=aSensorInfo->checker(PPFPGA);
    info()<< " bunch counter: [" << PPFPGA << "] " << bunchCounter <<endmsg;
    info()<< " detector ID: [" << PPFPGA << "] " << detectorID <<endmsg;
    info()<< " magic value: [" << PPFPGA << "] " << std::hex << check
          << std::dec <<endmsg;
    }
  //
  // get ADC data for given sensor and for each ALink
  dataVec ALink;
  for(int aLink=0; aLink<NumberOfALinks; aLink++){
    ALink=(*aSensorData)[aLink];
    // do analysis
  }
  // get adc headers for the sensor
  VeloTELL1Data* aSensorHeaders=m_decodedHeaders->object(aSensorData->key());
  dataVec headers;
  for(int aLink=0; aLink<NumberOfALinks; aLink++){
    headers=(*aSensorHeaders)[aLink];
    // do analysis
  }
  //
  return ( StatusCode::SUCCESS );
}
//
