// $Id: $
// Include files 

// from Gaudi
#include "GaudiKernel/DeclareFactoryEntries.h" 
#include "GaudiAlg/Tuples.h"

// local
#include "PrepareVeloFullRawBuffer.h"
#include "DecodeConf.h"

// data model
#include "Event/RawEvent.h"
#include "Event/RawBank.h"

// VeloDet
#include "VeloDet/DeVelo.h"
#include "VeloDet/DeVeloSensor.h"

//-----------------------------------------------------------------------------
// Implementation file for class : PrepareVeloFullRawBuffer
//
// 2006-03-22 : Tomasz Szumlak
//-----------------------------------------------------------------------------

// Declaration of the Algorithm Factory
DECLARE_ALGORITHM_FACTORY( PrepareVeloFullRawBuffer );

//=============================================================================
// Standard constructor, initializes variables
//=============================================================================
PrepareVeloFullRawBuffer::PrepareVeloFullRawBuffer( const std::string& name,
                                                  ISvcLocator* pSvcLocator)
  : GaudiTupleAlg ( name , pSvcLocator ),
    m_veloDet ( ),
    m_veloADCDataContainer ( VeloFullBankLocation::Default ),
    m_veloPedestalsContainer ( VeloFullBankLocation::Pedestals ),
    m_adcBankPresent ( false ),
    m_pedBankPresent ( false )
{
  declareProperty("FirstChannel", m_firstChannel=-4);
  declareProperty("LastChannel", m_lastChannel=2054);
  declareProperty("ADCLowLimit", m_ADCLowLimit=400);
  declareProperty("ADCHighLimit", m_ADCHighLimit=600);
  declareProperty("ChannelBins", m_channelBins=2058);
  declareProperty("ADCBins", m_ADCBins=200);
}
//=============================================================================
// Destructor
//=============================================================================
PrepareVeloFullRawBuffer::~PrepareVeloFullRawBuffer() {}
//=============================================================================
// Initialization
//=============================================================================
StatusCode PrepareVeloFullRawBuffer::initialize() {
  StatusCode sc = GaudiAlgorithm::initialize(); // must be executed first
  if ( sc.isFailure() ) return sc;  // error printed already by GaudiAlgorithm

  debug() << "==> Initialize" << endmsg;
  //
  m_veloDet=getDet<DeVelo>( DeVeloLocation::Default );
  setHistoTopDir( "Vetra/" );
  return StatusCode::SUCCESS;
}
//=============================================================================
// Main execution
//=============================================================================
StatusCode PrepareVeloFullRawBuffer::execute() {

  debug() << "==> Execute" << endmsg;
  //  
  StatusCode sc=getRawEvent();
  if(sc){
    getRawBanks();
    // if there is adc bank or pedestal bank begin ordering
    // for all read-out sensors
    if(adcBankFlag()||pedBankFlag()){
      int readedSensors=numberOfSensors();
      for(int sensor=0; sensor<readedSensors; sensor++){
        createOrderedSections(sensor);
      }
      writeVeloFull();
      // flush the vectors with pointers 
      // to the data bank after each event
      resetMemory();
    }
  }else{
    return ( StatusCode::FAILURE );
  }
  //
  return StatusCode::SUCCESS;
}
//=============================================================================
//  Finalize
//=============================================================================
StatusCode PrepareVeloFullRawBuffer::finalize() {

  debug() << "==> Finalize" << endmsg;

  return GaudiAlgorithm::finalize();  // must be called after all other actions
}
//=============================================================================
StatusCode PrepareVeloFullRawBuffer::getRawEvent()
{
  debug()<< " ==> getRawBuffer() " <<endmsg;
  debug()<< "--------------------" <<endmsg;
  //
  if(!exist<LHCb::RawEvent>(LHCb::RawEventLocation::Default)){
    error()<< " ==> There is no RawEvent at: "
           << LHCb::RawEventLocation::Default <<endmsg;
    return ( StatusCode::FAILURE );
  }else{  
    // get the RawEvent from default TES location
    m_rawEvent=get<LHCb::RawEvent>(LHCb::RawEventLocation::Default);
    debug()<< " ==> The RawEvent has been read-in from location: "
           << LHCb::RawEventLocation::Default <<endmsg;  
  }
  //
  return ( StatusCode::SUCCESS );
}
//=============================================================================
// /// ///////////////////////////////////////////////////////////////// /// //
// The getRawBanks() method is meant to check if non-zero suppresed data     //
// banks and pedestal banks are present in the read-in RawEvent, if this is  //
// the case the additional information needed to decoding are retrieved.     //
// For the VeloFull bank (non-zero data) pointers to the beggining of the    //
// data body of the bank are put into m_data container. Also, numbers of     //
// the read-out sensors (TELL1s) are stored inside m_sensors vector this     //
// allows to keep both number of read-out sensors and their IDs.             //
// The same procedure is repeated for VeloPedestal banks if they are         //
// detected (beside of caching the information about the sensors).           //
// /// ///////////////////////////////////////////////////////////////// /// //
StatusCode PrepareVeloFullRawBuffer::getRawBanks()
{
  debug()<< " ==> getRawBanks() " <<endmsg;
  // check if there is non-zero suppressed bank present
  const std::vector<LHCb::RawBank*>& fullBanks=
        m_rawEvent->banks(LHCb::RawBank::VeloFull);
  std::vector<LHCb::RawBank*>::const_iterator bIt;
  // if so get all info needed to decoding
  if(fullBanks.size()!=0){
    setNumberOfSensors(fullBanks.size());
    setADCBankFlag();
    m_veloADCData=new VeloFullBanks();
    //
    info()<< "VeloFull bank detected of size: " << fullBanks.size() <<endmsg;
    for(bIt=fullBanks.begin(); bIt!=fullBanks.end(); bIt++){
      LHCb::RawBank* aBank=(*bIt);
      // get the sensor number == sourceID
      int sensor=aBank->sourceID();
      debug()<< " sensor number: " << sensor <<endmsg;
      m_sensors.push_back(sensor);
      // get pointer to the beginning of data body of the bank
      unsigned int* data=aBank->data();
      debug()<< " ptr data[0] " << data <<endmsg;
      m_data.push_back(data);
    }
  }
  // check if there is pedestal bank present
  const std::vector<LHCb::RawBank*>& pedBanks=
    m_rawEvent->banks(LHCb::RawBank::VeloPedestal);
  // if so get informations about it
  if(pedBanks.size()!=0){
    setPedBankFlag();
    m_veloPedestals=new VeloFullBanks();
    //
    info()<< "VeloPed bank detected of size: " << pedBanks.size() <<endmsg;
    for(bIt=pedBanks.begin(); bIt!=pedBanks.end(); bIt++){
      LHCb::RawBank* aBank=(*bIt);
      // get pointer to the bank data
      unsigned int* data=aBank->data();
      debug()<< "Pointer to ped bank: " << data << " of number: "
            << aBank->sourceID() <<endmsg;
      m_ped.push_back(data);
    }
  }
  //
  if(m_data.size()==0) return ( StatusCode::FAILURE );
  //
  return ( StatusCode::SUCCESS );
}
//=============================================================================
void PrepareVeloFullRawBuffer::setNumberOfSensors(int sensors)
{
  debug()<< " ==> setNumberOfTELL1s() " <<endmsg;
  //
  m_numberOfSensors=sensors;
  //
  return;
}
//=============================================================================
int PrepareVeloFullRawBuffer::numberOfSensors()
{
  debug()<< " ==> numberOfTELL1s() " <<endmsg;
  //
  return ( m_numberOfSensors );
}
//=============================================================================
// /// ///////////////////////////////////////////////////////////////// /// //
// if there are both banks type order the data and create new data object    //
// and pass it to the next stage of the processing                           //
// /// ///////////////////////////////////////////////////////////////// /// //
void PrepareVeloFullRawBuffer::createOrderedSections(int nbOfBank)
{
  debug()<< " ==> separateSectors() " <<endmsg;
  //
  const int source=sourceID(nbOfBank);
  //
  if(adcBankFlag()){
    const int type_1=LHCb::RawBank::VeloFull;
    //
    VeloFullBank* aFullBank=new VeloFullBank(source, data(nbOfBank), type_1);
    m_veloADCData->insert(aFullBank);
  }
  //
  if(pedBankFlag()){
    const int type_2=LHCb::RawBank::VeloPedestal;
    VeloFullBank* aPedBank=new VeloFullBank(source, pedestals(nbOfBank), type_2);
    m_veloPedestals->insert(aPedBank);
  }
}
//=============================================================================
StatusCode PrepareVeloFullRawBuffer::writeVeloFull()
{
  debug()<< " ==> writeVeloFull() " <<endmsg;
  //
  if(adcBankFlag()){
    put(m_veloADCData, m_veloADCDataContainer);
    info()<< "Register container with bank data with size: "
          << m_veloADCData->size() << ", at" 
          << m_veloADCDataContainer <<endmsg;
  }
  //  
  if(pedBankFlag()){
    put(m_veloPedestals, m_veloPedestalsContainer);
    info()<< "Register container with bank data with size: "
          << m_veloPedestals->size() << ", at" 
          << m_veloPedestalsContainer <<endmsg;
  }
  //
  return ( StatusCode::SUCCESS );
}
//=============================================================================
int PrepareVeloFullRawBuffer::sourceID(int nbOfBank)
{
  debug()<< " ==> sourceID() " <<endmsg;
  //
  return ( m_sensors[nbOfBank] );
}
//=============================================================================
void PrepareVeloFullRawBuffer::resetMemory()
{
  debug()<< " ==> resetMemory() " <<endmsg;
  //
  m_data.clear();
  m_sensors.clear();
  m_ped.clear();
}
//=============================================================================
void PrepareVeloFullRawBuffer::dumpADCs(const dataVec& inADCs)
{
  info()<< " Dumping ADC values for all channels (1 TELL1)" <<endmsg;
  //
  cdatIt adcIt;
  int ii=0;
  for(adcIt=inADCs.begin(); adcIt!=inADCs.end(); adcIt++, ii++){
    if(!(ii%32)){
      info()<< " Beggining dumping ALink number: " << ii/32 <<endmsg;
    }
    //
    std::cout<< "[Beetle, Channel] = " << "[" << ii/128 << ", "
             << ii%32 << "]" << (*adcIt) <<std::endl;
  }
}
//=============================================================================
unsigned int* PrepareVeloFullRawBuffer::data(int noOfSensor)
{
  return ( m_data[noOfSensor] );
}
//=============================================================================
unsigned int* PrepareVeloFullRawBuffer::pedestals(int noOfSensor)
{
  return ( m_ped[noOfSensor] );
}
//=============================================================================
void PrepareVeloFullRawBuffer::setADCBankFlag()
{
  m_adcBankPresent=true;
}
//=============================================================================
void PrepareVeloFullRawBuffer::setPedBankFlag()
{
  m_pedBankPresent=true;
}
//=============================================================================
bool PrepareVeloFullRawBuffer::adcBankFlag()
{
  return ( m_adcBankPresent );
}
//=============================================================================
bool PrepareVeloFullRawBuffer::pedBankFlag()
{
  return ( m_pedBankPresent );
}
//
